SystemOrganization addCategory: #GODCall!

GODData subclass: #GCConference
	instanceVariableNames: 'name acronym edition categories url importantDates venue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCConference class methodsFor: 'initialize' stamp: 'pfsm 9/28/2014 16:17'!
name: conferenceName acronym: conferenceAcronym url: conferenceUrl what: conferenceCategories when: conferenceImportantDates where: conferenceVenue
	"constructs a GCConference object with all its fields"
	| conference |

	conference := self new.
	conference 	name: conferenceName;
						acronym: conferenceAcronym;
						url: conferenceUrl;
						categories: conferenceCategories;
						importantDates: conferenceImportantDates;
						venue: conferenceVenue.
	^ conference! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
acronym
	^ acronym! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
acronym: conferenceAcronym
	acronym := conferenceAcronym! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
categories
	^ categories! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
categories: conferenceCategories
	categories := conferenceCategories! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 10/12/2014 22:10'!
edition

	^ edition! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 10/12/2014 22:11'!
edition: anEdition
	"anEdition is usually the year"
	edition := anEdition! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
importantDates
	 ^ importantDates! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
importantDates: conferenceImportantDates
	importantDates := conferenceImportantDates! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
name
	^ name! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
name: conferenceName
	name := conferenceName! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 16:07'!
url
	^ url! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 16:08'!
url: aUrl

	url:= aUrl! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
venue
	 ^ venue! !

!GCConference methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 21:23'!
venue: conferenceVenue
	venue := conferenceVenue! !

Object subclass: #GCCrawler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCCrawler class methodsFor: 'parsing' stamp: 'pfsm 10/12/2014 22:55'!
parseWikiCfpUrl: categoryUrl
	"categoryUrl is the general url to the webpage of WikiCFP containing the conferences of a given category. This method returns a list of GCConferences whose attributes are extracted from categoryUrl"
	| pageNumber pageData emptyPage conferencesUrls filteredStrings categoryStartMark categoryEndMark conferencesList acronymAndEditionSMark acronymAndEditionEMark nameSMark nameEMark urlSMark urlEMark startAndDeadlineSMark startAndDeadlineEMark endSMark endEMark cityAndCountrySMark cityAndCountryEMark conference conferenceImportantDates conferenceVenue|


	filteredStrings := ''.
	emptyPage := false.
	pageNumber:=1.
	conferencesUrls := OrderedCollection new.
	categoryStartMark := '<td rowspan="2" align="left"><a href="'.
	categoryEndMark := '"'.

	"this loop fills in conferenceUrls with the links to each conference shown in the webpage pointed by categoryUrl "
	[emptyPage = false]
		whileTrue: [ 	pageData := GWDownloader getFromUrl: (((categoryUrl asString),'&page=', (pageNumber asString)) asUrl ).
							"filteredStrings := FILTConferenceFilter filter: (pageData content) startMark: categoryStartMark endMark: categoryEndMark."
							(filteredStrings size > 0)
								ifTrue: [	
												filteredStrings do: [:conferenceUrlSuffix | conferencesUrls add: ('http://www.wikicfp.com',conferenceUrlSuffix) asUrl].
												pageNumber := pageNumber+1.
											]
								ifFalse:[emptyPage := true ].
						].


	acronymAndEditionSMark :='"dc:title" content="'.
	acronymAndEditionEMark :='"'.
	nameSMark :='"dc:description" content="'.
	nameEMark :='"'.
	urlSMark :='"dc:source" content="'.
	urlEMark :='"'.
	startAndDeadlineSMark :='"v:startDate" content="'.
	startAndDeadlineEMark :='T'.
	endSMark :='"v:endDate" content="'.
	endEMark :='T'.
	cityAndCountrySMark :='"v:locality" content="'.
	cityAndCountryEMark :='"'.

	conferencesList := OrderedCollection new.
	conferencesUrls do:[ :confUrl | 	pageData := GWDownloader getFromUrl: confUrl.

													conference := GCConference new.
											
													"filteredStrings := FILTConferenceFilter filter: (pageData content) startMark: acronymAndEditionSMark endMark: acronymAndEditionEMark."
													filteredStrings := (filteredStrings at:1) findTokens: Character space.
													conference acronym: (filteredStrings at:1).
													conference edition: (filteredStrings at:2).

													"filteredStrings := FILTConferenceFilter filter: (pageData content) startMark: nameSMark endMark: nameEMark."
													conference name: (filteredStrings at:1).

													"filteredStrings := FILTConferenceFilter filter: (pageData content) startMark: urlSMark endMark: urlEMark."
													conference url: ((filteredStrings at:1) asUrl).

													conferenceImportantDates := GCImportantDates new.
													
													"filteredStrings := FILTConferenceFilter filter: (pageData content) startMark: startAndDeadlineSMark endMark: startAndDeadlineEMark."
													conferenceImportantDates startDate: ((filteredStrings at:1) asDate).
													conferenceImportantDates deadlineDate: ((filteredStrings at:2) asDate).

													"filteredStrings := FILTConferenceFilter filter: (pageData content) startMark: endSMark endMark: endEMark."
													conferenceImportantDates endDate: ((filteredStrings at:1) asDate).

													conferenceVenue := GCVenue new.
													
													"filteredStrings := FILTConferenceFilter filter: (pageData content) startMark: cityAndCountrySMark endMark: cityAndCountryEMark."
													filteredStrings := (filteredStrings at:1) findTokens: Character space.
													conferenceVenue city: ((filteredStrings at: 1) copyReplaceAll: ',' with: '').
													"sometimes there is only the city of the event"
 													conferenceVenue country: ((filteredStrings size > 1) 
																							ifTrue: [(filteredStrings at: 2)] 
																							ifFalse:['']).

													conference importantDates: conferenceImportantDates.
													conference venue: conferenceVenue.

													conferencesList add: conference.
								].
^ conferencesList.
	! !

!GCCrawler class methodsFor: 'initialize' stamp: 'cms 10/11/2014 23:42'!
quallisFileAsSpreadsheet
	"We wait a method from the spreadsheet group to get this input"

	 ^ self notYetImplemented.! !

!GCCrawler class methodsFor: 'initialize' stamp: 'cms 10/15/2014 00:33'!
run
		" search for conferences in web, merge information with quallis, and save in database"
	| sourcesCollection urlCollection page spreadsheetCollection spreadsheet mergedSpreadsheet godConference godConferenceCollection|

	sourcesCollection := GCSourceCreator createAll.
     
     	sourcesCollection do: [ :source |
   		urlCollection := source listUrls: (source categories) .
	      urlCollection do: [ :url |
			"page := GODWebPageFetcher getUrl: url.
			godConferenceCollection = source parseListPage: page
			godConferenceCollection addAll: godConference"
		]
		"spreadsheet := GODOutputSpreadsheet generateSpreadsheetFrom: godConferenceCollection.
		spreadsheetCollection add: spreadsheet.."
     	].

	spreadsheet :=  self getQuallisFileAsSpreadsheet.
	"mergedSpreadsheet = GODSpreadsheet  mergeSpreadsheets: spreadsheetCollection.
      TODO: usar grupo de processadores
	godConferenceCollection = GCProcessor generateGodConferencesFrom: mergedSpreadsheet.
	GODDBInput save: godConferenceCollection."
 	  ^ True.
! !

Object subclass: #GCImportantDates
	instanceVariableNames: 'startDate endDate deadlineDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCImportantDates class methodsFor: 'initialize' stamp: 'pfsm 9/28/2014 16:24'!
from: aStartDate to: anEndDate deadline: aDeadlineDate
	"constructs a GCImportantDates object"
	|importantDates|
	
	importantDates := self new.
	importantDates 	startDate: aStartDate;
							endDate: anEndDate;
							deadlineDate: aDeadlineDate.

	^ importantDates
! !

!GCImportantDates methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:21'!
deadlineDate
	^ deadlineDate! !

!GCImportantDates methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:22'!
deadlineDate: conferenceDeadlineDate
	deadlineDate := conferenceDeadlineDate! !

!GCImportantDates methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:19'!
endDate
	^ endDate! !

!GCImportantDates methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:19'!
endDate: conferenceEndDate
	endDate := conferenceEndDate! !

!GCImportantDates methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:18'!
startDate
	^ startDate! !

!GCImportantDates methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:19'!
startDate: conferenceStartDate
	startDate := conferenceStartDate! !

Object subclass: #GCSearchFilter
	instanceVariableNames: 'searchKey category startDate endDate deadlineDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCSearchFilter class methodsFor: 'initialize' stamp: 'pfsm 9/28/2014 11:58'!
new
	"creates a new object with all fields nil"
	| aSearchFilter |

	aSearchFilter := super new.
	aSearchFilter searchKey: nil;
						category: nil;
						startDate: nil;
						endDate: nil;
						deadlineDate: nil.

	^ aSearchFilter! !

!GCSearchFilter class methodsFor: 'initialize' stamp: 'pfsm 9/28/2014 12:02'!
search: aSearchKey in: aCategory from: aStartDate to: anEndDate
	"constructs a new searchFilter with the corresponding arguments "
	|aSearchFilter|

	aSearchFilter := self new.
	aSearchFilter searchKey: aSearchKey;
						category: aCategory;
						startDate: aStartDate;
						endDate: anEndDate.

	 ^  aSearchFilter
		! !

!GCSearchFilter class methodsFor: 'initialize' stamp: 'pfsm 9/28/2014 12:05'!
search: aSearchKey in: aCategory from: aStartDate to: anEndDate deadline: aDeadlineDate  
	"constructs a new searchFilter with the corresponding arguments "
	|aSearchFilter|

	aSearchFilter := self new.
	aSearchFilter searchKey: aSearchKey;
						category: aCategory;
						startDate: aStartDate;
						endDate: anEndDate;
						deadlineDate: aDeadlineDate.

	 ^  aSearchFilter
		! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:21'!
category

	^ category! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:32'!
category: aCategory
"set category of the conference being searched"
	category := aCategory! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:21'!
deadlineDate

	^ deadlineDate! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:31'!
deadlineDate: aDeadlineDate
"set filter for searching conferences with deadline until aDeadlineDate"
	deadlineDate := aDeadlineDate! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:21'!
endDate

	^ endDate! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:32'!
endDate: anEndDate
"set filter for searching conferences with end date until anEndDate"
	endDate := anEndDate! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'fabcm 10/14/2014 00:13'!
printString
	"Create a string describing the contents stored on search filter"

	^ self notYetImplemented.! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:21'!
searchKey

	^ searchKey! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:34'!
searchKey: aSearchKey
"set filter for searching conferences that contain the word aSearchKey"
	searchKey := aSearchKey! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:21'!
startDate

	^ startDate! !

!GCSearchFilter methodsFor: 'accessing' stamp: 'pfsm 9/28/2014 10:35'!
startDate: aStartDate
"set filter for searching conferences with start date after  aStartDate"
	startDate := aStartDate! !

Object subclass: #GCSource
	instanceVariableNames: 'categoryFilepath parser baseListUrl'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCSource methodsFor: 'acessing' stamp: 'cms 10/14/2014 23:44'!
baseListUrl: anBaseUrlList
	baseListUrl := anBaseUrlList.! !

!GCSource methodsFor: 'acessing' stamp: 'cms 10/15/2014 00:30'!
categories
	"returns all categories found in categoryFilepath. Each category must occupy one line in the file"
	|fileData categories|
	fileData := (StandardFileStream fileNamed: categoryFilepath ) contentsOfEntireFile.
	categories:= fileData lines.
	^ categories.! !

!GCSource methodsFor: 'acessing' stamp: 'cms 10/14/2014 23:36'!
categoryFilepath
	 ^ categoryFilepath.! !

!GCSource methodsFor: 'acessing' stamp: 'cms 10/14/2014 23:41'!
categoryFilepath: aFilepath
	"set path to file containing categories of this source"
	categoryFilepath := aFilepath.! !

!GCSource methodsFor: 'acessing' stamp: 'cms 10/14/2014 23:46'!
listUrls: categories
	|urlList|
	urlList := OrderedCollection new.
	categories do: [ :category |
		category := category copyReplaceAll: ' ' with: '%20'.
		urlList add: ((baseListUrl, category) asUrl).
	].
	^urlList.! !

!GCSource methodsFor: 'acessing' stamp: 'cms 10/14/2014 23:41'!
parser
	^ parser.! !

!GCSource methodsFor: 'acessing' stamp: 'cms 10/14/2014 23:41'!
parser: aParser
	parser := aParser.! !

Object subclass: #GCSourceCreator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCSourceCreator class methodsFor: 'create sources' stamp: 'cms 10/14/2014 23:59'!
createAll
	|all|
	all := Bag new.	
	all add: (self createConfSearch).
	all add: (self createWikiCfp).
	^ all.! !

!GCSourceCreator class methodsFor: 'create sources' stamp: 'cms 10/15/2014 00:06'!
createConfSearch
	"creates and returns a instance of GCSource that represents the source WikiCfp"
	|source|
	source := GCSource new.
	source baseListUrl: 'http://confsearch.org/confsearch/faces/pages/topic.jsp?topic='.
	source categoryFilepath: '/home/camila/Dropbox/POO 2014/confSearch_categories.txt'.
	source parser: GCConfSearchParser new.
	^ source.
! !

!GCSourceCreator class methodsFor: 'create sources' stamp: 'cms 10/15/2014 00:07'!
createWikiCfp
	"creates and returns a instance of GCSource that represents the source WikiCfp"
	|source|
	source := GCSource new.
	source baseListUrl: 'http://www.wikicfp.com/cfp/call?conference='.
	source categoryFilepath: '/home/phablo/Dropbox/POO 2014/wikiCfp_categories.txt'.
	source parser: GCWikiCfpParser new.
	^ source.! !

Object subclass: #GCSourceParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

GCSourceParser subclass: #GCConfSearchParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCConfSearchParser methodsFor: 'parsing' stamp: 'cms 10/15/2014 00:05'!
parseDetailPage: page
	^ GCConference new.! !

!GCConfSearchParser methodsFor: 'parsing' stamp: 'cms 10/15/2014 00:06'!
parseListPage: page
	^ Array new.! !

!GCSourceParser methodsFor: 'parsing' stamp: 'cms 10/15/2014 00:03'!
parseDetailPage: page
	"given a page with information of one conference this method returns the respective conference as object"
	^ self subclassResponsibility.! !

!GCSourceParser methodsFor: 'parsing' stamp: 'cms 10/15/2014 00:01'!
parseListPage: page
	"given a page with a list of conferences this method returns a collection of GCConference"
	^ self subclassResponsibility.! !

GCSourceParser subclass: #GCWikiCfpParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCWikiCfpParser methodsFor: 'parsing' stamp: 'cms 10/15/2014 00:05'!
parseDetailPage: page
	^ GCConference new.! !

!GCWikiCfpParser methodsFor: 'parsing' stamp: 'cms 10/15/2014 00:06'!
parseListPage: page
	^ Array new.! !

Object subclass: #GCVenue
	instanceVariableNames: 'place city country'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GCVenue class methodsFor: 'initialize' stamp: 'pfsm 9/28/2014 16:29'!
place: aPlace city: aCity country: aCountry 
	"constucts a GCVenue object"
	| venue |

	venue:= self new.
	venue	place: aPlace;
				city: aCity;
				country: aCountry.

	^ venue! !

!GCVenue methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:25'!
city
	^ city! !

!GCVenue methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:25'!
city: conferenceCity
	city := conferenceCity! !

!GCVenue methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:25'!
country
	^ country! !

!GCVenue methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:25'!
country: conferenceCountry
	country := conferenceCountry! !

!GCVenue methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:25'!
place
	^ place! !

!GCVenue methodsFor: 'accessing' stamp: 'pfsm 9/27/2014 12:26'!
place: conferencePlace
	place := conferencePlace! !

Object subclass: #GODCall
	instanceVariableNames: 'categories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODCall'!

!GODCall methodsFor: 'accessing' stamp: 'pfsm 10/12/2014 01:25'!
categories

	^ categories! !

!GODCall methodsFor: 'accessing' stamp: 'pfsm 10/12/2014 01:25'!
categories: anObject

	categories := anObject! !

!GODCall methodsFor: 'initialize' stamp: 'cms 10/5/2014 20:24'!
crawlConferences
   ^ GCCrawler run.     
	! !

!GODCall methodsFor: 'interface' stamp: 'fabcm 10/13/2014 22:44'!
godCallIndexPage
	"THIS METHOD SHOULD BE REMOVED"! !

!GODCall methodsFor: 'initialize' stamp: 'fabcm 10/13/2014 22:36'!
initialize
	"crawlConferences for the first time and register GODCall to Seaside"

	self crawlConferences.
	WEBAppManager register: 'God Call' to: self.! !

!GODCall methodsFor: 'interface' stamp: 'fabcm 10/14/2014 00:38'!
renderContent
	"Create the first page of GodCall with a formulary for the user define his search preferences."

	| page welcome body list filter categoryForm deadlineForm startForm endForm searchForm showFilter submit |

	page := GODData new.
	page layoutGridRowSize: 4 columnSize: 1.

	welcome := GODData new.
	welcome title: 'Welcome to Gods Call!!'.
	welcome content: 'Please, choose below your search preferences'.
	page addOnLayoutGrid: welcome at: 1 at: 1.

	body := GODData new. "body will have a dropdown list together with a inputform associated with the item chosen in the list"
	body layoutGridRowSize: 1 columnSize: 2.
	page addOnLayoutGrid: body at: 2 at: 1.

	filter := GCSearchFilter	new.

	categoryForm := WEBInputForm value: 'Category:' callback: [ :input | filter category: input. ].
	deadlineForm := WEBInputForm value: 'Deadline Date:' callback: [ :input | filter deadlineDate: input. ]. 
	"Maybe in the dates input we need to convert the string to a date..."
	startForm := WEBInputForm value: 'Start Date:' callback: [ :input | filter startDate: input. ].
	endForm := WEBInputForm value: 'End Date:' callback: [ :input | filter endDate: input. ].
	searchForm := WEBInputForm value: 'Search Key:' callback: [ :input | filter searchKey: input. ].

	list := WEBDropdownList values: #( 'Category' 'Deadline Date' 'Start Date' 'End Date' 'Search Key'  ) 
							   callback: [ :input | ( input = 'Category') ifTrue: [ body addOnLayoutGrid: categoryForm at: 1 at: 2.].
												( input = 'Deadline Date') ifTrue: [ body addOnLayoutGrid: deadlineForm at: 1 at: 2.].
												( input = 'Start Date') ifTrue: [ body addOnLayoutGrid: startForm at: 1 at: 2.].
												( input = 'End Date') ifTrue: [ body addOnLayoutGrid: endForm at: 1 at: 2.].
												( input = 'Search Key') ifTrue: [ body addOnLayoutGrid: searchForm at: 1 at: 2.]. ].
	body addOnLayoutGrid: list at: 1at: 1.

	showFilter := GODData new.
	showFilter title: 'Your Search Parameters:'.
	showFilter content: filter printString.
	page addOnLayoutGrid: showFilter at: 3 at: 1.
	
	submit := WEBInputForm value: 'Click here to search!!' callback: [ :input | ^ self renderOutputPage: (GCProcessor submitForm: filter). ]. "Here, I just wanted a button..."
	page addOnLayoutGrid: submit at: 4 at: 1.

	^ page.! !

!GODCall methodsFor: 'interface' stamp: 'fabcm 10/14/2014 00:26'!
renderOutputPage
	"THIS METHOD SHOULD BE REMOVED"! !

!GODCall methodsFor: 'interface' stamp: 'fabcm 10/14/2014 00:24'!
renderOutputPage: conferencesSpreadsheet
	"Create the output page with the conferences."

	 ^ self notYetImplemented.! !

!GODCall methodsFor: 'interface' stamp: 'fabcm 10/14/2014 00:37'!
submitForm: searchFilter
	"THIS METHOD SHOULD BE REMOVED"! !
